@@ -1,5 +1,5 @@
 package App::ansifold;
-our $VERSION = "1.1503";
+our $VERSION = "1.16";
 
 use v5.14;
 use warnings;
@@ -11,6 +11,8 @@ use Pod::Usage;
 use List::Util qw(min);
 use Hash::Util qw(lock_keys);
 use Text::ANSI::Fold qw(:constants);
+use Text::ANSI::Fold::Util qw(ansi_width);
+use Unicode::EastAsianWidth;
 use Data::Dumper;
 
 our $DEFAULT_WIDTH    //= 72;
@@ -28,25 +30,27 @@ use Getopt::EX::Hashed 'has'; {
 	$_->padding = 1;
 	$_->padchar = $_[1] if $_[1] ne '';
     };
-    has padchar   => '   =s  ' ;
-    has prefix    => '   =s  ' ;
-    has ambiguous => '   =s  ' ;
-    has paragraph => ' p +   ' , default => 0;
-    has separate  => '   =s  ' , default => $DEFAULT_SEPARATE;
-    has linebreak => '   =s  ' , alias   => 'lb';
-    has runin     => '   =i  ' , min => 1, default => 4;
-    has runout    => '   =i  ' , min => 1, default => 4;
-    has nonewline => ' n     ' ;
-    has smart     => ' s !   ' ;
-    has expand    => ' x :-1 ' , default => $DEFAULT_EXPAND;
-    has tabstop   => '   =i  ' , min => 1;
-    has tabhead   => '   =s  ' ;
-    has tabspace  => '   =s  ' ;
-    has tabstyle  => '   =s  ' ;
-    has discard   => '   =s@ ' , default => [];
-    has colrm     => '       ' , default => $DEFAULT_COLRM;
-    has help      => ' h     ' ;
-    has version   => ' v     ' ;
+    has padchar    => '   =s  ' ;
+    has prefix     => '   =s  ' ;
+    has autoindent => '   =s  ' ;
+    has ambiguous  => '   =s  ' ;
+    has paragraph  => ' p +   ' , default => 0;
+    has refill     => ' r +   ' , default => 0;
+    has separate   => '   =s  ' , default => $DEFAULT_SEPARATE;
+    has linebreak  => '   =s  ' , alias   => 'lb';
+    has runin      => '   =i  ' , min => 1, default => 4;
+    has runout     => '   =i  ' , min => 1, default => 4;
+    has nonewline  => ' n     ' ;
+    has smart      => ' s !   ' ;
+    has expand     => ' x :-1 ' , default => $DEFAULT_EXPAND;
+    has tabstop    => '   =i  ' , min => 1;
+    has tabhead    => '   =s  ' ;
+    has tabspace   => '   =s  ' ;
+    has tabstyle   => '   =s  ' ;
+    has discard    => '   =s@ ' , default => [];
+    has colrm      => '       ' , default => $DEFAULT_COLRM;
+    has help       => ' h     ' ;
+    has version    => ' v     ' ;
 
     has '+boundary'  => any => [ qw(none word space) ];
     has '+ambiguous' => any => [ qw(wide narrow) ] ;
@@ -91,7 +95,9 @@ use Getopt::EX::Hashed 'has'; {
 	};
     };
 
+    # internal use
     has width_index => default => [];
+    has indent_pat => ;
 
 } no Getopt::EX::Hashed;
 
@@ -141,6 +147,10 @@ sub options {
 	$_ = charnames::string_vianame($_) || die "$_: invalid name\n";
     }
 
+    if (my $indent = $app->autoindent) {
+	$app->indent_pat = qr/$indent/;
+    }
+
     return $app;
 }
 
@@ -200,18 +210,35 @@ sub doit {
 
     my @index = @{$app->width_index};
 
+    local $/ = '' if $app->refill;
     while (<>) {
 	my $chomped = chomp;
+	fill_paragraph() if $app->refill;
+	my @opt;
+	if ($app->{indent_pat} && /^$app->{indent_pat}/p) {
+	    my $indent = ansi_width ${^MATCH};
+	    my $prefix = ' ' x $indent;
+	    $fold->configure(prefix => $prefix);
+	}
 	my @chops = $fold->text($_)->chops;
 	@chops = grep { defined } @chops[@index] if @index > 0;
 	print join $separator, @chops;
-	print "\n" if $chomped;
+	print "\n" x $chomped if $chomped;
 	print "\n" x $app->paragraph if $app->paragraph > 0;
     }
 
     return $app;
 }
 
+sub fill_paragraph {
+    s{
+	(?<full> (?<=\p{InFullwidth})\n(?=\p{InFullwidth}) ) |
+	(?<half> [ ]*\n[ ]* )
+    }{
+	$+{full} ? '' : ' '
+    }xge;
+}
+
 sub terminal_width {
     use Term::ReadKey;
     my $default = 80;
